
CREATE INDEX reference_ubvri_loc_index ON reference_ubvri
       USING btree( ra, dec );
CREATE INDEX reference_ubvri_star_id_index ON reference_ubvri
       USING btree( star_id );
CREATE INDEX new_reference_loc_index ON new_reference_loc
       USING btree( ra, dec );

CREATE INDEX files_range_index ON files
       USING btree( ra_min, ra_max, dec_min, dec_max );
CREATE INDEX files_id_index ON files USING btree( file_id, group_id, pair_id );

CREATE INDEX group_id_index ON groups USING btree( group_id );
CREATE INDEX pairs_id_index ON pairs USING btree( pair_id, group_id );

CREATE INDEX fits_id_index ON fits USING btree( file_id  );

CREATE INDEX zero_pair_pair_id_index ON zero_pair USING btree( pair_id );
CREATE INDEX color_group_group_id_index ON color_group USING btree( group_id );

CREATE INDEX catalog_ra_decl_index ON catalog USING btree( ra, dec );

CREATE INDEX obs_u_obs_id_index ON obs_u USING btree( obs_id );
CREATE INDEX obs_b_obs_id_index ON obs_b USING btree( obs_id );
CREATE INDEX obs_v_obs_id_index ON obs_v USING btree( obs_id );
CREATE INDEX obs_r_obs_id_index ON obs_r USING btree( obs_id );
CREATE INDEX obs_i_obs_id_index ON obs_i USING btree( obs_id );

CREATE INDEX temp_obs_u_obs_id_index ON temp_obs_u USING btree( obs_id );
CREATE INDEX temp_obs_b_obs_id_index ON temp_obs_b USING btree( obs_id );
CREATE INDEX temp_obs_v_obs_id_index ON temp_obs_v USING btree( obs_id );
CREATE INDEX temp_obs_r_obs_id_index ON temp_obs_r USING btree( obs_id );
CREATE INDEX temp_obs_i_obs_id_index ON temp_obs_i USING btree( obs_id );

CREATE INDEX obs_u_use_index ON obs_u USING btree( use );
CREATE INDEX obs_b_use_index ON obs_b USING btree( use );
CREATE INDEX obs_v_use_index ON obs_v USING btree( use );
CREATE INDEX obs_r_use_index ON obs_r USING btree( use );
CREATE INDEX obs_i_use_index ON obs_i USING btree( use );

CREATE INDEX obs_u_star_id_index ON obs_u USING btree( star_id );
CREATE INDEX obs_b_star_id_index ON obs_b USING btree( star_id );
CREATE INDEX obs_v_star_id_index ON obs_v USING btree( star_id );
CREATE INDEX obs_r_star_id_index ON obs_r USING btree( star_id );
CREATE INDEX obs_i_star_id_index ON obs_i USING btree( star_id );

CREATE INDEX obs_u_file_id_index ON obs_u USING btree( file_id );
CREATE INDEX obs_b_file_id_index ON obs_b USING btree( file_id );
CREATE INDEX obs_v_file_id_index ON obs_v USING btree( file_id );
CREATE INDEX obs_r_file_id_index ON obs_r USING btree( file_id );
CREATE INDEX obs_i_file_id_index ON obs_i USING btree( file_id );

CREATE INDEX temp_obs_u_file_id_index ON temp_obs_u USING btree( file_id );
CREATE INDEX temp_obs_b_file_id_index ON temp_obs_b USING btree( file_id );
CREATE INDEX temp_obs_v_file_id_index ON temp_obs_v USING btree( file_id );
CREATE INDEX temp_obs_r_file_id_index ON temp_obs_r USING btree( file_id );
CREATE INDEX temp_obs_i_file_id_index ON temp_obs_i USING btree( file_id );

CREATE INDEX obs_u_loc_index ON obs_u USING btree( ra, dec );
CREATE INDEX obs_b_loc_index ON obs_b USING btree( ra, dec );
CREATE INDEX obs_v_loc_index ON obs_v USING btree( ra, dec );
CREATE INDEX obs_r_loc_index ON obs_r USING btree( ra, dec );
CREATE INDEX obs_i_loc_index ON obs_i USING btree( ra, dec );

CREATE INDEX temp_obs_u_loc_index ON temp_obs_u USING btree( ra, dec );
CREATE INDEX temp_obs_b_loc_index ON temp_obs_b USING btree( ra, dec );
CREATE INDEX temp_obs_v_loc_index ON temp_obs_v USING btree( ra, dec );
CREATE INDEX temp_obs_r_loc_index ON temp_obs_r USING btree( ra, dec );
CREATE INDEX temp_obs_i_loc_index ON temp_obs_i USING btree( ra, dec );

CREATE INDEX welch_index_updated_index ON welch_index USING btree( updated );
CREATE INDEX welch_index_star_id_index ON welch_index USING btree( star_id );
