#!/usr/bin/perl 

use warnings;
use strict;
use Shell qw(psql);

psql( qw/-d tassiv -f dbSchema.sql/ );
psql( qw/-d tassiv -f dbIndexes.sql/ );
psql( qw/-d tassiv -f dbTriggers.sql/ );

system( qw:dbLoadTycho2.pl /tass/catalogs/Tycho-2/data/catalog.dat: );
system( qw:dbSeedubvri.pl: );
