--
-- $Id: tassiv_triggers.sql,v 1.3 2002/02/20 05:20:14 robc Exp $
--
-- $Name:  $
--
-- $Log: tassiv_triggers.sql,v $
-- Revision 1.3  2002/02/20 05:20:14  robc
-- *** empty log message ***
--

DROP TRIGGER obs_u_trig ON obs_u;
DROP TRIGGER obs_b_trig ON obs_b;
DROP TRIGGER obs_v_trig ON obs_v;
DROP TRIGGER obs_r_trig ON obs_r;
DROP TRIGGER obs_i_trig ON obs_i;

DROP FUNCTION observations_trigger( );

CREATE FUNCTION observations_trigger( ) RETURNS OPAQUE AS
'/usr/local/pgsql/triggers/tassiv_triggers.so' LANGUAGE 'C';

CREATE TRIGGER obs_u_trig BEFORE INSERT OR UPDATE OR DELETE
   ON obs_u FOR EACH ROW EXECUTE PROCEDURE observations_trigger( );
CREATE TRIGGER obs_b_trig BEFORE INSERT OR UPDATE OR DELETE
   ON obs_b FOR EACH ROW EXECUTE PROCEDURE observations_trigger( );
CREATE TRIGGER obs_v_trig BEFORE INSERT OR UPDATE OR DELETE
   ON obs_v FOR EACH ROW EXECUTE PROCEDURE observations_trigger( );
CREATE TRIGGER obs_r_trig BEFORE INSERT OR UPDATE OR DELETE
   ON obs_r FOR EACH ROW EXECUTE PROCEDURE observations_trigger( );
CREATE TRIGGER obs_i_trig BEFORE INSERT OR UPDATE OR DELETE
   ON obs_i FOR EACH ROW EXECUTE PROCEDURE observations_trigger( );
